@@ -1,5 +1,5 @@
 /* This file is part of Patchage.
- * Copyright 2008-2014 David Robillard <http://drobilla.net>
+ * Copyright 2008-2020 David Robillard <d@drobilla.net>
  *
  * Patchage is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free
@@ -15,28 +15,46 @@
  */
 
 #ifndef _GNU_SOURCE
-	#define _GNU_SOURCE
+#	define _GNU_SOURCE
 #endif
 
-#include <assert.h>
-#include <limits.h>
-#include <stdlib.h>
 #include <dlfcn.h>
 
-/** Return the absolute path of the binary.
- * Returned value must be freed by caller.
- */
-static char*
+#include <climits>
+#include <cstdlib>
+#include <string>
+
+namespace patchage {
+
+/** Return the absolute path of the binary. */
+inline std::string
 binary_location()
 {
-	Dl_info dli;
-	const int ret = dladdr((void*)&binary_location, &dli);
-	if (ret) {
-		char* const bin_loc = (char*)calloc(PATH_MAX, 1);
-		if (!realpath(dli.dli_fname, bin_loc)) {
-			return NULL;
-		}
-		return bin_loc;
+	Dl_info   dli = {};
+	const int ret = dladdr(reinterpret_cast<void*>(&binary_location), &dli);
+	if (!ret) {
+		return "";
 	}
-	return NULL;
+
+	char* const bin_loc = realpath(dli.dli_fname, nullptr);
+	if (!bin_loc) {
+		return "";
+	}
+
+	std::string loc{bin_loc};
+	free(bin_loc);
+	return loc;
 }
+
+/** Return the absolute path of the bundle (binary parent directory). */
+inline std::string
+bundle_location()
+{
+	const std::string binary = binary_location();
+	if (binary.empty()) {
+		return "";
+	}
+	return binary.substr(0, binary.find_last_of('/'));
+}
+
+} // namespace patchage
